@@ -2,8 +2,8 @@
   This file is part of usb_modeswitch, a mode switching tool for controlling
   flip flop (multiple device) USB gear
 
-  Version 1.2.4, 2012/08/12
-  Copyright (C) 2007 - 2012  Josua Dietze
+  Version 2.0.1, 2013/09/03
+  Copyright (C) 2007 - 2013  Josua Dietze
 
   Config file parsing stuff borrowed from Guillaume Dargaud
   (http://www.gdargaud.net/Hack/SourceCode.html)
@@ -22,9 +22,8 @@
 
 */
 
-
 #include <stdlib.h>
-#include <usb.h>
+#include <libusb.h>
 
 void readConfigFile(const char *configFilename);
 void printConfig();
@@ -32,27 +31,30 @@ int switchSendMessage();
 int switchConfiguration();
 int switchAltSetting();
 void switchHuaweiMode();
+
 void switchSierraMode();
 void switchGCTMode();
-int switchKobilMode();
-int switchQisdaMode();
-int switchSequansMode();
-int switchActionMode();
+void switchKobilMode();
+void switchQisdaMode();
+void switchQuantaMode();
+void switchSequansMode();
+void switchActionMode();
+void switchBlackberryMode();
+void switchPantechMode();
+void switchCiscoMode();
 int switchSonyMode();
-int switchCiscoMode();
 int detachDriver();
 int checkSuccess();
 int sendMessage(char* message, int count);
 int write_bulk(int endpoint, char *message, int length);
 int read_bulk(int endpoint, char *buffer, int length);
 void release_usb_device(int dummy);
-struct usb_device* search_devices( int *numFound, int vendor, int product, char* productList,
-	int targetClass, int configuration, int mode);
-int find_first_bulk_output_endpoint(struct usb_device *dev);
-int find_first_bulk_input_endpoint(struct usb_device *dev);
-int get_current_configuration(struct usb_dev_handle* devh);
-int get_interface0_class(struct usb_device *dev, int devconfig);
-int get_interface_class(struct usb_device *dev, int cfgNumber, int ifcNumber);
+struct libusb_device* search_devices( int *numFound, int vendor, char* productList,
+		int targetClass, int configuration, int mode);
+int find_first_bulk_output_endpoint(struct libusb_device *dev);
+int find_first_bulk_input_endpoint(struct libusb_device *dev);
+int get_current_configuration(struct libusb_device_handle* devh);
+int get_interface_class(struct libusb_config_descriptor *cfg, int ifcNumber);
 char* ReadParseParam(const char* FileName, char *VariableName);
 int hex2num(char c);
 int hex2byte(const char *hex);
@@ -64,6 +66,8 @@ void deviceDescription();
 int deviceInquire();
 void resetUSB();
 void release_usb_device(int dummy);
+int findMBIMConfig(int vendor, int product, int mode);
+
 
 // Boolean
 #define  and     &&
@@ -106,3 +110,7 @@ extern char *TempPP;
 	if ((TempPP=ReadParseParam((ParamFileName), #B))!=NULL) \
 		B=(toupper(TempPP[0])=='Y' || toupper(TempPP[0])=='T'|| TempPP[0]=='1'); else B=0
 
+#define ParseParamBoolMap(ParamFileName, B, M, Const) \
+	if ((TempPP=ReadParseParam((ParamFileName), #B))!=NULL) \
+		if (toupper(TempPP[0])=='Y' || toupper(TempPP[0])=='T'|| TempPP[0]=='1') \
+			M=M+Const
