@@ -9,8 +9,8 @@
 # the mode switching program with the matching parameter
 # file from /usr/share/usb_modeswitch
 #
-# Part of usb-modeswitch-1.2.4 package
-# (C) Josua Dietze 2009-2012
+# Part of usb-modeswitch-2.0.1 package
+# (C) Josua Dietze 2009-2013
 
 set arg0 [lindex $argv 0]
 if [regexp {\.tcl$} $arg0] {
@@ -26,16 +26,16 @@ if [regexp {\.tcl$} $arg0] {
 
 set flags(logging) 0
 set flags(noswitching) 0
-
-#set env(PATH) "/bin:/sbin:/usr/bin:/usr/sbin"
+set flags(stordelay) 0
 
 # Execution starts at file bottom
 
 proc {Main} {argv argc} {
 
-global scsi usb config match device flags settings
+global scsi usb config match device flags setup devdir loginit
 
-set loginit [ParseGlobalConfig]
+set flags(logwrite) 0
+Log "[ParseGlobalConfig]"
 
 # The facility to add a symbolic link pointing to the
 # ttyUSB port which provides interrupt transfer, i.e.
@@ -52,37 +52,44 @@ if {[lindex $argv 0] == "--symlink-name"} {
 	SafeExit
 }
 
-set argList [split [lindex $argv 1] /]
-
-if [string length [lindex $argList 1]] {
-	set device [lindex $argList 1]
+if {[lindex $argv 0] == "--switch-systemd"} {
+	set device [string trim [lindex $argv 1] "/-"]
+	set device [regsub {/} $device "-"]
+	set argList [list "" $device]
+	Log "\nStarted via systemd"
 } else {
-	set device "noname"
+	if {[lindex $argv 0] == "--switch-upstart"} {
+		Log "\nStarted via upstart"
+	}
+	set argList [split [lindex $argv 1] /]
+	if [string length [lindex $argList 1]] {
+		set device [lindex $argList 1]
+	} else {
+		set device "noname"
+	}
+}
+if {$flags(stordelay) > 0} {
+	SetStorageDelay $flags(stordelay)
 }
 
-Log "Raw args from udev: [lindex $argv 1]\n\n$loginit"
+Log "Raw args from udev: [lindex $argv 1]\n"
 
 if {$device == "noname"} {
-	Log "No data from udev. Exiting"
+	Log "\nNo data from udev. Exit"
 	SafeExit
 }
 
-if {[lindex $argv 0] != "--switch-mode"} {
-	Log "No command given. Exiting"
+if {![regexp -- {--switch-} [lindex $argv 0]]} {
+	Log "\nNo command given. Exit"
 	SafeExit
 }
 
-if {![regexp /lib/udev/usb_modeswitch [lindex $argv 2]]} {
-	Log "Dispatcher was not run from call script. Exiting"
-	SafeExit
-}
-
-set settings(dbdir)	/usr/share/usb_modeswitch
-set settings(dbdir_etc)	/etc/usb_modeswitch.d
+set setup(dbdir) /usr/share/usb_modeswitch
+set setup(dbdir_etc) /etc/usb_modeswitch.d
 
 
-if {![file exists $settings(dbdir)] && ![file exists $settings(dbdir_etc)]} {
-	Log "Error: no config database found in /usr/share or /etc. Exiting"
+if {![file exists $setup(dbdir)] && ![file exists $setup(dbdir_etc)]} {
+	Log "\nError: no config database found in /usr/share or /etc. Exit"
 	SafeExit
 }
 set bindir /usr/sbin
@@ -91,23 +98,25 @@ set devList1 {}
 set devList2 {}
 
 
-# arg 0: the bus id for the device (udev: %b)
+# arg 0: the bus id for the device (udev: %b), often ommitted
 # arg 1: the "kernel name" for the device (udev: %k)
 #
-# Both together give the top directory where the path
-# to the SCSI attributes can be determined (further down)
-# Addendum: older kernel/udev version seem to differ in
-# providing these attributes - or not. So more probing
-# is needed
+# Used to determine the top directory for the device in sysfs
 
+set ifChk 0
 if {[string length [lindex $argList 0]] == 0} {
 	if {[string length [lindex $argList 1]] == 0} {
-		Log "No device number values given from udev! Exiting"
+		Log "No device number values given from udev! Exit"
 		SafeExit
 	} else {
 		if {![regexp {(.*?):} [lindex $argList 1] d dev_top]} {
-			Log "Could not determine top device dir from udev values! Exiting"
-			SafeExit
+			if [regexp {([0-9]+-[0-9]+\.?[0-9]*.*)} [lindex $argList 1] d dev_top] {
+				# new udev rules file, got to check class of first interface
+				set ifChk 1
+			} else {
+				Log "Could not determine device dir from udev values! Exit"
+				SafeExit
+			}
 		}
 	}
 } else {
@@ -117,11 +126,31 @@ if {[string length [lindex $argList 0]] == 0} {
 
 set devdir /sys/bus/usb/devices/$dev_top
 if {![file isdirectory $devdir]} {
-	Log "Top device directory not found ($devdir)! Exiting"
+	Log "Top device directory not found ($devdir)! Exit"
 	SafeExit
 }
-Log "Using top device dir $devdir"
-set ifdir "[file tail $devdir]:1.0"
+Log "Use top device dir $devdir"
+
+set iface 0
+if $ifChk {
+	Log "Check class of first interface ..."
+	set iclass [IfClass 0]
+	if {$iface < 0} {
+		Log " No access to interface 0. Exit"
+		SafeExit
+	}
+	if {$iclass == 8} {
+		Log " Device is in install mode."
+	} else {
+		# No install mode, so no logging at all
+		exit
+	}
+}
+set ifdir [file tail [IfDir $iface]]
+regexp {:([0-9]+\.[0-9]+)$} $ifdir d iface
+
+set flags(logwrite) 1
+Log "Use interface $iface"
 
 
 # Mapping of the short string identifiers (in the config
@@ -139,12 +168,12 @@ set match(uSe) usb(serial)
 
 # Now reading the USB attributes
 if {![ReadUSBAttrs $devdir]} {
-	Log "USB attributes not found in sysfs tree. Exiting"
+	Log "USB attributes not found in sysfs tree. Exit"
 	SafeExit
 }
 
 if $flags(logging) {
-	Log "----------------\nUSB values from sysfs:"
+	Log "\n----------------\nUSB values from sysfs:"
 	foreach attr {manufacturer product serial} {
 		Log "  $attr\t$usb($attr)"
 	}
@@ -152,8 +181,8 @@ if $flags(logging) {
 }
 
 if $flags(noswitching) {
-	Log "\nSwitching globally disabled. Exiting\n"
 	SysLog "usb_modeswitch: switching disabled, no action for $usb(idVendor):$usb(idProduct)"
+	Log "\nSwitching globally disabled. Exit"
 	SafeExit
 }
 
@@ -171,7 +200,7 @@ if {$usb(bNumConfigurations) == "1"} {
 set configList [ConfigGet conflist $usb(idVendor):$usb(idProduct)]
 
 if {[llength $configList] == 0} {
-	Log "Aargh! Config file missing for $usb(idVendor):$usb(idProduct)! Exiting"
+	Log "Aargh! Config file missing for $usb(idVendor):$usb(idProduct)! Exit"
 	SafeExit
 }
 
@@ -182,7 +211,7 @@ if {[llength $configList] > 1} {
 	}
 }
 if $scsiNeeded {
-	if [ReadSCSIAttrs $devdir:1.0] {
+	if [ReadSCSIAttrs $devdir:$iface] {
 		Log "----------------\nSCSI values from sysfs:"
 		foreach attr {vendor model rev} {
 			Log " $attr\t$scsi($attr)"
@@ -192,7 +221,7 @@ if $scsiNeeded {
 		Log "Could not get SCSI attributes, exclude devices with SCSI match"
 	}
 } else {
-	Log "SCSI attributes not needed, moving on"
+	Log "SCSI attributes not needed, move on"
 }
 
 # General wait - this is important
@@ -207,9 +236,9 @@ foreach configuration $configList {
 	# skipping installer leftovers
 	if [regexp {\.(dpkg|rpm)} $configuration] {continue}
 
-	Log "checking config: $configuration"
+	Log "Check config: $configuration"
 	if [MatchDevice $configuration] {
-		Log "! matched. Reading config data"
+		Log "! matched. Read config data"
 		if [string length $usb(busnum)] {
 			set busParam "-b [string trimleft $usb(busnum) 0]"
 			set devParam "-g [string trimleft $usb(devnum) 0]"
@@ -221,16 +250,37 @@ foreach configuration $configList {
 		ParseDeviceConfig $configBuffer
 		if {$config(waitBefore) == ""} {
 		} else {
-			Log " waiting time set to $config(waitBefore) seconds"
+			Log " delay time set to $config(waitBefore) seconds"
 			append config(waitBefore) "000"
 			after $config(waitBefore)
-			Log " waiting is over, switching starts now"
+			Log " wait is over, start mode switch"
+		}
+		if {$config(noMBIMCheck)==0 && $usb(bNumConfigurations) > 1} {
+			Log "Device may have an MBIM configuration, check driver ..."
+			if [CheckMBIM] {
+				Log " driver for MBIM devices is available"
+				Log "Find MBIM configuration number ..."
+				if [catch {set cfgno [exec /usr/sbin/usb_modeswitch -j -Q $busParam $devParam -v $usb(idVendor) -p $usb(idProduct)]} err] {
+					Log "Error when trying to find MBIM configuration, switch to legacy modem mode"
+				} else {
+					set cfgno [string trim $cfgno]
+					if {$cfgno > 0} {
+						set config(Configuration) $cfgno
+						set config(driverModule) ""
+						set configBuffer "Configuration=$cfgno"
+					} else {
+						Log " No MBIM configuration found, switch to legacy modem mode"
+					}
+				}
+			} else {
+				Log " no MBIM driver found, switch to legacy modem mode"
+			}
 		}
 
 		# Now we are actually switching
 		if $flags(logging) {
-			Log "Command to be run:\nusb_modeswitch -I -W -D -s 20 $configParam $busParam $devParam -v $usb(idVendor) -p $usb(idProduct) -f \$configBuffer"
-			set report [exec /usr/sbin/usb_modeswitch -I -W -D -s 20 $configParam $busParam $devParam -v $usb(idVendor) -p $usb(idProduct) -f "$configBuffer" 2>@ stdout]
+			Log "Command to be run:\nusb_modeswitch -W -D -s 20 $configParam $busParam $devParam -v $usb(idVendor) -p $usb(idProduct) -f \$configBuffer"
+			set report [exec /usr/sbin/usb_modeswitch -W -D -s 20 $configParam $busParam $devParam -v $usb(idVendor) -p $usb(idProduct) -f "$configBuffer" 2>@1]
 			Log "\nVerbose debug output of usb_modeswitch and libusb follows"
 			Log "(Note that some USB errors are to be expected in the process)"
 			Log "--------------------------------"
@@ -238,7 +288,7 @@ foreach configuration $configList {
 			Log "--------------------------------"
 			Log "(end of usb_modeswitch output)\n"
 		} else {
-			set report [exec /usr/sbin/usb_modeswitch -I -Q -D -s 20 $configParam $busParam $devParam -v $usb(idVendor) -p $usb(idProduct) -f "$configBuffer" 2>@ stdout]
+			set report [exec /usr/sbin/usb_modeswitch -Q -D -s 20 $configParam $busParam $devParam -v $usb(idVendor) -p $usb(idProduct) -f "$configBuffer" 2>@1]
 		}
 		break
 	} else {
@@ -260,16 +310,16 @@ if [regexp {ok:busdev} $report] {
 		foreach attr [lsort [array names usb]] {
 			Log "    [format %-26s $attr:] $usb($attr)"
 		}
-		Log "\nMode switching may have failed. Exiting\n"
+		Log "\nMode switching may have failed. Exit"
 		SafeExit
 	}
 } else {
 	if {![file isdirectory $devdir]} {
-		Log "Device directory in sysfs is gone! Something went wrong, aborting"
+		Log "Device directory in sysfs is gone! Something went wrong, abort"
 		SafeExit
 	}
 	if {![regexp {ok:} $report]} {
-		Log "\nCore program reported switching failure. Exiting\n"
+		Log "\nCore program reported switching failure. Exit"
 		SafeExit
 	}
 	# Give the device another second if it's not fully back yet
@@ -281,10 +331,10 @@ if [regexp {ok:busdev} $report] {
 
 # Now checking for bound drivers (only for class 0xff)
 
-if {$usb($ifdir/bInterfaceClass) != "" && [regexp {ok:} $report]} {
+if {$config(driverModule) != "" && $usb($ifdir/bInterfaceClass) != "" && [regexp {ok:} $report]} {
 	if {$usb($ifdir/bInterfaceClass) != "ff"} {
 		set config(driverModule) ""
-		Log " No vendor-specific class found, skip driver checking"
+		Log " No vendor-specific class found, skip driver check"
 	}
 }
 
@@ -294,13 +344,13 @@ if {$usb($ifdir/bInterfaceClass) != "" && [regexp {ok:} $report]} {
 if {$config(driverModule) != ""} {
 	if {[string length "$usb(idVendor)$usb(idProduct)"] < 8} {
 		if {![regexp {ok:(\w{4}):(\w{4})} $report d usb(idVendor) usb(idProduct)]} {
-			Log "No target vendor/product ID found or given, can't continue. Aborting"
+			Log "No target vendor/product ID found or given, can't continue. Abort"
 			SafeExit
 		}
 	}
 	# wait for any drivers to bind automatically
 	after 1000
-	Log "Now checking for bound driver ..."
+	Log "Now check for bound driver ..."
 	if {![file exists $devdir/$ifdir/driver]} {
 		Log " no driver has bound to interface 0 yet"
 		AddToList link_list $usb(idVendor):$usb(idProduct)
@@ -339,14 +389,14 @@ if {$config(driverModule) != ""} {
 
 if [regexp {ok:$} $report] {
 	# "NoDriverLoading" was set
-	Log "Doing no driver checking or binding for this device"
+	Log "No driver check or bind for this device"
 }
 
 # In newer kernels there is a switch to avoid the use of a device
 # reset (e.g. from usb-storage) which would possibly switch back
 # a mode-switching device to initial mode
 if [regexp {ok:} $report] {
-	Log "Checking for AVOID_RESET_QUIRK kernel attribute"
+	Log "Check for AVOID_RESET_QUIRK kernel attribute"
 	if [file exists $devdir/avoid_reset_quirk] {
 		if [catch {exec echo "1" >$devdir/avoid_reset_quirk 2>/dev/null} err] {
 			Log " Error setting the attribute: $err"
@@ -358,7 +408,7 @@ if [regexp {ok:} $report] {
 	}
 }
 
-Log "\nAll done, exiting\n"
+Log "\nAll done, exit\n"
 SafeExit
 
 }
@@ -370,7 +420,7 @@ proc {ReadSCSIAttrs} {topdir} {
 global scsi
 set counter 0
 set sysdir $topdir
-Log "Checking storage tree in sysfs ..."
+Log "Check storage tree in sysfs ..."
 while {$counter < 20} {
 	Log " loop $counter/20"
 	if {![file isdirectory $sysdir]} {
@@ -402,7 +452,7 @@ if {$counter == 20} {
 	return 0
 }
 
-Log "Reading SCSI values ..."
+Log "Read SCSI values ..."
 foreach attr {vendor model rev} {
 	if [file exists $sysdir/$attr] {
 		set rc [open $sysdir/$attr r]
@@ -466,9 +516,9 @@ foreach teststring $stringList {
 	set matchstring [lindex $tokenList 1]
 	set blankstring ""
 	regsub -all {_} $matchstring { } blankstring
-	Log "matching $match($id)"
-	Log "  match string1 (exact):  $matchstring"
-	Log "  match string2 (blanks): $blankstring"
+	Log "match $match($id)"
+	Log "  string1 (exact):  $matchstring"
+	Log "  string2 (blanks): $blankstring"
 	Log " device string: [set $match($id)]"
 	if {!([string match *$matchstring* [set $match($id)]] || [string match *$blankstring* [set $match($id)]])} {
 		return 0
@@ -496,6 +546,7 @@ if {$configFile == ""} {return}
 set rc [open $configFile r]
 while {![eof $rc]} {
 	gets $rc line
+	if [regexp {^#} [string trim $line]] {continue}
 	if [regexp {DisableSwitching\s*=\s*([^\s]+)} $line d val] {
 		if [regexp -nocase {1|yes|true} $val] {
 			set flags(noswitching) 1
@@ -506,9 +557,14 @@ while {![eof $rc]} {
 			set flags(logging) 1
 		}
 	}
+	if [regexp {SetStorageDelay\s*=\s*([^\s]+)} $line d val] {
+		if [regexp {\d+} $val] {
+			set flags(stordelay) $val
+		}
+	}
 
 }
-return "Using global config file: $configFile"
+return "Use global config file: $configFile"
 
 }
 # end of proc {ParseGlobalConfig}
@@ -524,6 +580,7 @@ set config(targetVendor) ""
 set config(targetProduct) ""
 set config(targetClass) ""
 set config(Configuration) ""
+set config(noMBIMCheck) 0
 set config(checkSuccess) 20
 set loadDriver 1
 
@@ -554,6 +611,9 @@ if [regexp -line {^[^#]*?CheckSuccess.*?=.*?([0-9]+).*?$} $configContent d confi
 if [regexp -line {^[^#]*?WaitBefore.*?=.*?([0-9]+).*?$} $configContent d config(waitBefore)] {
 	Log "config: WaitBefore set to $config(waitBefore)"
 }
+if [regexp -line {^[^#]*?NoMBIMCheck.*?=.*?([0-9]+).*?$} $configContent d config(noMBIMCheck)] {
+	Log "config: noMBIMCheck set to $config(noMBIMCheck)"
+}
 if [regexp -line {^[^#]*?NoDriverLoading.*?=.*?(1|yes|true).*?$} $configContent] {
 	set loadDriver 0
 	Log "config: NoDriverLoading is set to active"
@@ -582,17 +642,17 @@ set config(waitBefore) [string trimleft $config(waitBefore) 0]
 
 proc ConfigGet {command config} {
 
-global settings
+global setup
 
 switch $command {
 
 	conflist {
 		# Unpackaged configs first; sorting is essential for priority
-		set configList [lsort -decreasing [glob -nocomplain $settings(dbdir_etc)/$config*]]
-		set configList [concat $configList [lsort -decreasing [glob -nocomplain $settings(dbdir)/$config*]]]
-		if [file exists $settings(dbdir)/configPack.tar.gz] {
-			Log "Found packed config collection $settings(dbdir)/configPack.tar.gz"
-			if [catch {set packedList [exec tar -tzf $settings(dbdir)/configPack.tar.gz 2>/dev/null]} err] {
+		set configList [lsort -decreasing [glob -nocomplain $setup(dbdir_etc)/$config*]]
+		set configList [concat $configList [lsort -decreasing [glob -nocomplain $setup(dbdir)/$config*]]]
+		if [file exists $setup(dbdir)/configPack.tar.gz] {
+			Log "Found packed config collection $setup(dbdir)/configPack.tar.gz"
+			if [catch {set packedList [exec tar -tzf $setup(dbdir)/configPack.tar.gz 2>/dev/null]} err] {
 				Log "Error: problem opening config package; tar returned\n $err"
 				return {}
 			}
@@ -609,12 +669,12 @@ switch $command {
 	conffile {
 		if [regexp {^pack/} $config] {
 			set config [regsub {pack/} $config {}]
-			Log "Extracting config $config from collection $settings(dbdir)/configPack.tar.gz"
-			set configContent [exec tar -xzOf $settings(dbdir)/configPack.tar.gz $config 2>/dev/null]
+			Log "Extract config $config from collection $setup(dbdir)/configPack.tar.gz"
+			set configContent [exec tar -xzOf $setup(dbdir)/configPack.tar.gz $config 2>/dev/null]
 		} else {
-			if [regexp [list $settings(dbdir_etc)] $config] {
-				Log "Using config file from override folder $settings(dbdir_etc)"
-				SysLog "usb_modeswitch: using overriding config file $config; make sure this is intended"
+			if [regexp [list $setup(dbdir_etc)] $config] {
+				Log "Use config file from override folder $setup(dbdir_etc)"
+				SysLog "usb_modeswitch: use overriding config file $config; make sure this is intended"
 				SysLog "usb_modeswitch: please report any new or corrected settings; otherwise, check for outdated files"
 			}
 			set rc [open $config r]
@@ -630,34 +690,34 @@ switch $command {
 
 proc {Log} {msg} {
 
-global flags device
-if {$flags(logging) == 0} {return}
+global flags device loginit
 
-if {![info exists flags(wc)]} {
-	if [catch {set flags(wc) [open /var/log/usb_modeswitch_$device w]} err] {
-		if [catch {set flags(wc) [open /dev/console w]} err] {
-			set flags(wc) "error"
-			return
-		} else {
-			puts $flags(wc) "Error opening log file ($err), redirect to console"
-		}
+if {$flags(logging) == 0} {return}
+#puts $msg
+#return
+
+if $flags(logwrite) {
+	if [string length $loginit] {
+		exec echo "\nUSB_ModeSwitch log from [clock format [clock seconds]]" >/var/log/usb_modeswitch_$device
+		exec echo "$loginit" >>/var/log/usb_modeswitch_$device
+#"
+		set loginit ""
 	}
-	puts $flags(wc) "\n\nUSB_ModeSwitch log from [clock format [clock seconds]]\n"
+	exec echo $msg >>/var/log/usb_modeswitch_$device
+} else {
+	append loginit "\n$msg"
 }
-if {$flags(wc) == "error"} {return}
-puts $flags(wc) $msg
 
 }
 # end of proc {Log}
 
 
-# Closing the log file if open and exit
+# Writing the log file and exit
 proc {SafeExit} {} {
 
 global flags
-if [info exists flags(wc)] {
-	catch {close $flags(wc)}
-}
+set $flags(logwrite) 1
+Log ""
 exit
 
 }
@@ -669,12 +729,12 @@ global device
 
 proc {hasInterrupt} {ifDir} {
 	if {[llength [glob -nocomplain $ifDir/ttyUSB*]] == 0} {
-		Log "  no ttyUSB interface - skip checking endpoints"
+		Log "  no ttyUSB interface - skip endpoint check"
 		return 0
 	}
 	foreach epDir [glob -nocomplain $ifDir/ep_*] {
 		set e [file tail $epDir]
-		Log "  checking $e ..."
+		Log "  check $e ..."
 		if [file exists $epDir/type] {
 			set rc [open $epDir/type r]
 			set type [read $rc]
@@ -698,7 +758,7 @@ if [file exists $linkpath] {
 		set rawpath [file readlink $linkpath]
 		set trimpath [regsub -all {\.\./} $rawpath {}]
  		if [file isdirectory /sys/$trimpath] {
-			append loginit "\n Using path $path\n"
+			append loginit "\n Use path $path\n"
 			set path /$trimpath
 		}
 	}
@@ -707,7 +767,7 @@ if [file exists $linkpath] {
 if {![regexp {ttyUSB[0-9]+} $path myPort]} {
 	if $flags(logging) {
 		set device [clock clicks]
-		Log "$loginit\nThis is not a ttyUSB port. Aborting"
+		Log "$loginit\nThis is not a ttyUSB port. Abort"
 	}
 	return ""
 }
@@ -716,13 +776,13 @@ set device $myPort
 Log "$loginit\nMy name is $myPort\n"
 
 if {![regexp {(.*?[0-9]+)\.([0-9]+)/ttyUSB} /sys$path d ifRoot ifNum]} {
-	Log "Could not find interface in path\n $path. Aborting"
+	Log "Could not find interface in path\n $path. Abort"
 	return ""
 }
 
 set ifDir $ifRoot.$ifNum
 
-Log "Checking my endpoints ...\n in $ifDir"
+Log "Check my endpoints ...\n in $ifDir"
 if [hasInterrupt $ifDir] {
 	Log "\n--> I am an interrupt port"
 	set rightPort 1
@@ -736,7 +796,7 @@ if [hasInterrupt $ifDir] {
 # possible lower interfaces
 
 if { $rightPort && ($ifNum > 0) } {
-	Log "\nLooking for lower ports with interrupt endpoints"
+	Log "\nLook for lower ports with interrupt endpoints"
 	for {set i 0} {$i < $ifNum} {incr i} {
 		set ifDir $ifRoot.$i
 		Log " in ifDir $ifDir ..."
@@ -793,7 +853,7 @@ if {![file exists $idfile]} {
 		Log "Can't do anymore without module loader; get \"modtools\"!"
 		return
 	}
-	Log "\nTrying to load module \"$config(driverModule)\""
+	Log "\nTry to load module \"$config(driverModule)\""
 	if [catch {set result [exec $loader -v $config(driverModule)]} err] {
 		Log " Running \"$loader $config(driverModule)\" gave an error:\n  $err"
 	} else {
@@ -811,8 +871,8 @@ while {$i < 50} {
 	incr i
 }
 if {$i < 50} {
-	Log "Trying to add ID to driver \"$config(driverModule)\""
-	SysLog "usb_modeswitch: adding device ID $vid:$pid to driver \"$config(driverModule)\""
+	Log "Try to add ID to driver \"$config(driverModule)\""
+	SysLog "usb_modeswitch: add device ID $vid:$pid to driver \"$config(driverModule)\""
 	SysLog "usb_modeswitch: please report the device ID to the Linux USB developers!"
 	if [catch {exec echo "$vid $pid ff" >$idfile} err] {
 		Log " Error adding ID to driver:\n  $err"
@@ -821,16 +881,16 @@ if {$i < 50} {
 	}
 } else {
 	Log " \"$idfile\" not found, check if kernel version is at least 2.6.27"
-	Log "Falling back to \"usbserial\""
+	Log "Fall back to \"usbserial\""
 	set config(driverModule) usbserial
-	Log "\nTrying to unload driver \"usbserial\""
+	Log "\nTry to unload driver \"usbserial\""
 	if [catch {exec $loader -r usbserial} err] {
 		Log " Running \"$loader -r usbserial\" gave an error:\n  $err"
 		Log "No more fallbacks"
 		return
 	}
 	after 50
-	Log "\nTrying to load driver \"usbserial\" with device IDs"
+	Log "\nTry to load driver \"usbserial\" with device IDs"
 	if [catch {set result [exec $loader -v usbserial vendor=0x$vid product=0x$pid]} err] {
 		Log " Running \"$loader usbserial\" gave an error:\n  $err"
 	} else {
@@ -923,25 +983,29 @@ close $lc
 }
 # end of proc {RemoveFromBindList}
 
+
 proc {CheckSuccess} {devdir} {
 
 global config usb
-set ifdir "[file tail $devdir]:1.0"
+set ifdir [file tail [IfDir 0]]
 
 if {[string length $config(targetClass)] || [string length $config(Configuration)]} {
 	set config(targetVendor) $usb(idVendor)
 	set config(targetProduct) $usb(idProduct)
 }
-Log "Checking success of mode switch for max. $config(checkSuccess) seconds ..."
+Log "Check success of mode switch for max. $config(checkSuccess) seconds ..."
 
 for {set i 1} {$i <= $config(checkSuccess)} {incr i} {
 	after 1000
 	if {![file isdirectory $devdir]} {
-		Log " Waiting for device file system ($i sec.) ..."
+		Log " Wait for device file system ($i sec.) ..."
 		continue
 	} else {
-		Log " Reading attributes ..."
+		Log " Read attributes ..."
 	}
+	set ifdir [IfDir 0]
+	if {$ifdir == ""} {continue}
+	set ifdir [file tail $ifdir]
 	if {![ReadUSBAttrs $devdir $ifdir]} {
 		Log " Essential attributes are missing, continue wait ..."
 		continue
@@ -950,7 +1014,7 @@ for {set i 1} {$i <= $config(checkSuccess)} {incr i} {
 		if {![regexp $usb($ifdir/bInterfaceClass) $config(targetClass)]} {continue}
 	}
 	if [string length $config(Configuration)] {
-		if {$usb(bConfigurationValue) != $config(Configuration} {continue}
+		if {$usb(bConfigurationValue) != $config(Configuration)} {continue}
 	}
 	if {![regexp $usb(idVendor) $config(targetVendor)]} {continue}
 	if {![regexp $usb(idProduct) $config(targetProduct)]} {continue}
@@ -965,6 +1029,40 @@ return 1
 }
 # end of proc {CheckSuccess}
 
+
+proc {IfDir} {iface} {
+
+global devdir
+set allfiles [glob -nocomplain $devdir/*]
+set files [glob -nocomplain $devdir/*.$iface]
+if {[llength $files] == 0} {
+	return ""
+}
+set ifdir [lindex $files 0]
+if {![file isdirectory $ifdir]} {
+	return ""
+}
+return $ifdir
+
+}
+# end of proc {IfDir}
+
+proc {IfClass} {iface} {
+
+set ifdir [IfDir $iface]
+
+if {![file exists $ifdir/bInterfaceClass]} {
+	return -1
+}
+set rc [open $ifdir/bInterfaceClass r]
+set c [read $rc]
+close $rc
+return [string trimleft [string trim $c] 0]
+
+}
+# end of proc {IfClass}
+
+
 proc {SysLog} {msg} {
 
 global flags
@@ -986,6 +1084,38 @@ catch {exec $flags(logger) -p syslog.notice "$msg" 2>/dev/null}
 }
 # end of proc {SysLog}
 
+proc {SetStorageDelay} {secs} {
+
+Log "Adjust delay for USB storage devices ..."
+set attrib /sys/module/usb_storage/parameters/delay_use
+if {![file exists $attrib]} {
+	Log "Error: could not find delay_use attribute"
+	return
+}
+if [catch {set ch [open $attrib r+]} err] {
+	Log "Error: could not access delay_use attribute: $err"
+	return
+}
+if {[read $ch] < $secs} {
+	seek $ch 0 start
+	puts -nonewline $ch $secs
+	Log " Delay set to $secs seconds\n"
+} else {
+	Log " Current value is higher than $secs. Leave it alone\n"
+}
+close $ch
+
+}
+# end of proc {SetStorageDelay}
+
+proc {CheckMBIM} {} {
+
+set kversion [exec uname -r]
+if [file exists /lib/modules/$kversion/kernel/drivers/net/usb/cdc_mbim.ko] {return 1}
+if [file exists /sys/bus/usb/drivers/cdc_mbim] {return 1}
+return 0
+
+}
 
 
 # The actual entry point
